# Install ActiveMQ, JDK, and Tomcat; Ignore PostgreSQL
Add-Type -AssemblyName System.IO.Compression.FileSystem

#Versions
$VSN_ACTIVEMQ           = "5.15.6"
$VSN_JDK                = "8.31.0.1-jdk8.0.181"
$VSN_TOMCAT             = "9.0.12"

#Names
$NAME_ACTIVEMQ          = "activemq"
$NAME_TOMCAT            = "tomcat"
$NAME_TOMCAT_LAUNCH     = "start-tomcat.bat"
$NAME_TOMCAT_QUIT       = "stop-tomcat.bat"
$NAME_TOMCAT_SERVICE    = "TomcatService.exe"

#Directories
$DIR_ACTIVEMQ           = "${NAME_ACTIVEMQ}"
$DIR_BACKUP             = "backup"
$DIR_BASE               = "c:\netiq\idm\apps"
$DIR_COMPRESSED         = "${PSScriptRoot}\compressed"
$DIR_JDK                = "jdk"
$DIR_LOG_BASE           = "${PSScriptRoot}"
$DIR_LOG_PARENT         = "logs"
$DIR_LOG                = "${DIR_LOG_BASE}\${DIR_LOG_PARENT}"
$DIR_PRISTINE           = "${PSScriptRoot}\untouched"
$DIR_TOMCAT             = "${NAME_TOMCAT}"

#Files
$FILE_ACTIVEMQ          = "${DIR_COMPRESSED}\apache-activemq-${VSN_ACTIVEMQ}-bin.zip"
$FILE_INDEX             = "${DIR_PRISTINE}\index.html"
$FILE_JDK               = "${DIR_COMPRESSED}\zulu${VSN_JDK}-win_x64.zip"
$FILE_LOG               = "${DIR_LOG}\IDGov35-Core-Helper-IDGov.log"
$FILE_TOMCAT            = "${DIR_COMPRESSED}\apache-tomcat-${VSN_TOMCAT}-windows-x64.zip"
$FILE_TOMCAT_LAUNCH     = "${DIR_PRISTINE}\${NAME_TOMCAT_LAUNCH}"
$FILE_TOMCAT_QUIT       = "${DIR_PRISTINE}\${NAME_TOMCAT_QUIT}"
$FILE_TOMCAT_SERVICE    = "${DIR_PRISTINE}\${NAME_TOMCAT_SERVICE}"

#Services
$SERV_TOMCAT_SERVICE    = "IDM Apps Tomcat Service"

#Parameters
$PARAM_OSP_LDAP_TYPE    = "edir"
$PARAM_OSP_LOGGING      = "WARN"
$PARAM_TOMCAT_SERVER    = "localhost"
$PARAM_TOMCAT_OPTS_COMMON_ALL = "-Dnetiqtomcatcustomsrvidentify=netiqtomcatcustomsrvidentify;-Djava.awt.headless=true;-Dcom.netiq.ism.config=${DIR_BASE}\${DIR_TOMCAT}\conf\ism-configuration.properties;"
$PARAM_TOMCAT_OPTS_OSP  = "-Dcom.netiq.osp.ext-context-file=${DIR_BASE}\osp\lib\osp-conf-${PARAM_OSP_LDAP_TYPE}.jar;-Dcom.netiq.idm.osp.logging.level=${PARAM_OSP_LOGGING};-Dcom.netiq.idm.osp.client.host=${PARAM_TOMCAT_SERVER};-Dcom.netiq.idm.osp.logging.file.dir=${DIR_BASE}\${DIR_TOMCAT}\logs;-Dcom.netiq.idm.osp.audit.enabled=false;"
$PARAM_TOMCAT_OPTS_COMMON_IDENT = "-Dlog4j.configuration=file:${DIR_BASE}\${DIR_TOMCAT}\conf\log4j.properties;-Dinternal.atlaslite.jcce.xml.w3c.XMLUtil.suppressSecurityWarning=true;"
$PARAM_TOMCAT_OPTS_GOV  = "-Dcom.netiq.ism.config.is.jndi.env=true;-Dnovell.logging.config.dir=${DIR_BASE}\${DIR_TOMCAT}\conf;"
$PARAM_TOMCAT_OPTS_RPT  = "-Dcom.netiq.rpt.config.file=${DIR_BASE}\${DIR_TOMCAT}\conf\ism-configuration.properties;-Dextend.local.config.dir=${DIR_BASE}\${DIR_TOMCAT}\conf;-Didmuserapp.logging.config.dir=${DIR_BASE}\${DIR_TOMCAT}\conf;"
$PARAM_TOMCAT_JVM_OPTS  = "${PARAM_TOMCAT_OPTS_COMMON_ALL}${PARAM_TOMCAT_OPTS_OSP}${PARAM_TOMCAT_OPTS_COMMON_IDENT}${PARAM_TOMCAT_OPTS_GOV}${PARAM_TOMCAT_OPTS_RPT}"
$PARAM_TOMCAT_JVM_MEM_INIT  = "1024"
$PARAM_TOMCAT_JVM_MEM_FINAL = "1024"


Function Unzip
{
    param([string]${ZIP_FILE}, [string]${OUT_DIR})

    [System.IO.Compression.ZipFile]::ExtractToDirectory(${ZIP_FILE}, ${OUT_DIR})
}


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Stopping existing transcripts"
Write-Host "######################################################################################"
$ErrorActionPreference="SilentlyContinue"
Stop-Transcript | out-null


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Creating log directory at: ${DIR_LOG}"
Write-Host "######################################################################################"
$null = (New-Item -Path "${DIR_LOG_BASE}" -Name "${DIR_LOG_PARENT}" -ItemType "directory")


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Removing previous log if exists: ${FILE_LOG}"
Write-Host "######################################################################################"
Remove-Item -Path "${FILE_LOG}"


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Starting transcript: ${FILE_LOG}"
Write-Host "######################################################################################"
$ErrorActionPreference = "Continue"
Start-Transcript -path "${FILE_LOG}" -append


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Extracting ActiveMQ to: ${DIR_BASE}\${DIR_ACTIVEMQ}"
Write-Host "######################################################################################"
Unzip "${FILE_ACTIVEMQ}" "${DIR_BASE}"
Rename-Item -Path "${DIR_BASE}\apache-activemq-${VSN_ACTIVEMQ}" -newName "${DIR_ACTIVEMQ}"


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Extracting JDK to: ${DIR_BASE}\${DIR_JDK}"
Write-Host "######################################################################################"
Unzip "${FILE_JDK}" "${DIR_BASE}"
Rename-Item -Path "${DIR_BASE}\zulu${VSN_JDK}-win_x64" -newName "${DIR_JDK}"


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Extracting Tomcat to: ${DIR_BASE}\${DIR_TOMCAT}"
Write-Host "######################################################################################"
Unzip "$FILE_TOMCAT" "${DIR_BASE}"
Rename-Item -Path "${DIR_BASE}\apache-tomcat-${VSN_TOMCAT}" -newName "${DIR_TOMCAT}"


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Copying the ActiveMQ Jar to: ${DIR_BASE}\${DIR_TOMCAT}\lib"
Write-Host "######################################################################################"
Copy-Item "${DIR_BASE}\${DIR_ACTIVEMQ}\${DIR_ACTIVEMQ}-all*.jar" -Destination "${DIR_BASE}\${DIR_TOMCAT}\lib"


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Moving the default Tomcat wars to: ${DIR_BASE}\${DIR_BACKUP}"
Write-Host "######################################################################################"
$null = (New-Item -Path "${DIR_BASE}" -Name "${DIR_BACKUP}" -ItemType "directory")
Get-ChildItem -Path "${DIR_BASE}\${DIR_TOMCAT}\webapps" -Recurse | Move-Item -Destination "${DIR_BASE}\${DIR_BACKUP}"


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Creating new default Tomcat index file at: ${DIR_BASE}\${DIR_TOMCAT}\webapps\ROOT\${FILE_INDEX}"
Write-Host "######################################################################################"
$null = (New-Item -Path "${DIR_BASE}\${DIR_TOMCAT}\webapps" -Name "ROOT" -ItemType "directory")
Copy-Item -Path "${FILE_INDEX}" -Destination "${DIR_BASE}\${DIR_TOMCAT}\webapps\ROOT\"


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Copy ${FILE_TOMCAT_SERVICE} to: ${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_SERVICE}"
Write-Host "######################################################################################"
Copy-Item -Path "${FILE_TOMCAT_SERVICE}" -Destination "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_SERVICE}"


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Copy ${FILE_TOMCAT_LAUNCH} to: ${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}"
Write-Host "######################################################################################"
Copy-Item -Path "${FILE_TOMCAT_LAUNCH}" -Destination "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}"


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Prepare: ${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}"
Write-Host "######################################################################################"
$text = (Get-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}" -ReadCount 0) -join "`r`n"
$text -replace "REPLACE_INSTALL_BASE", "${DIR_BASE}" | Set-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}"
$text = (Get-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}" -ReadCount 0) -join "`r`n"
$text -replace "REPLACE_INSTALL_JAVA", "${DIR_BASE}\${DIR_JDK}" | Set-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}"


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Copy ${FILE_TOMCAT_QUIT} to: ${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}"
Write-Host "######################################################################################"
Copy-Item -Path "${FILE_TOMCAT_QUIT}" -Destination "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}"


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Prepare: ${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}"
Write-Host "######################################################################################"
$text = (Get-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}" -ReadCount 0) -join "`r`n"
$text -replace "REPLACE_INSTALL_BASE", "${DIR_BASE}" | Set-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}"
$text = (Get-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}" -ReadCount 0) -join "`r`n"
$text -replace "REPLACE_INSTALL_JAVA", "${DIR_BASE}\${DIR_JDK}" | Set-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}"


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Create the Tomcat service: ${SERV_TOMCAT_SERVICE}"
Write-Host "######################################################################################"
New-Service -Name "${SERV_TOMCAT_SERVICE}" -BinaryPathName "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_SERVICE} ${DIR_BASE}" -StartupType Manual


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Restarting the Tomcat service using: ${SERV_TOMCAT_SERVICE}"
Write-Host "######################################################################################"
Restart-Service -Name "${SERV_TOMCAT_SERVICE}"


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Stop the Tomcat service using: ${SERV_TOMCAT_SERVICE}"
Write-Host "######################################################################################"
Stop-Service -Name "${SERV_TOMCAT_SERVICE}"


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Complete the transcript"
Write-Host "######################################################################################"
Stop-Transcript


Write-Host ""
Write-Host "######################################################################################"
Write-Host " Completed"
Write-Host " Review the log file: ${FILE_LOG}"
Write-Host "######################################################################################"
